#!/home/twinkle/venv/bin/python
# -*- encoding: utf-8 -*-

import gi

gi.require_version('Gtk', '3.0')
gi.require_version('Gdk', '3.0')

from gi.repository import Gtk
from gi.repository import Gdk # GDK_SELECTION_CLIPBOARD
#from gi.repository import Pango

######################################################################
# VARS

MessageType = {
     'GTK_MESSAGE_INFO': 0,
  'GTK_MESSAGE_WARNING': 1,
 'GTK_MESSAGE_QUESTION': 2,
    'GTK_MESSAGE_ERROR': 3,
    'GTK_MESSAGE_OTHER': 4,
}

ButtonType = {
      'GTK_BUTTONS_NONE': 0,
        'GTK_BUTTONS_OK': 1,
     'GTK_BUTTONS_CLOSE': 2,
    'GTK_BUTTONS_CANCEL': 3,
    'GTK_BUTTONS_YES_NO': 4,
 'GTK_BUTTONS_OK_CANCEL': 5,
}

ResponseType = {
         'GTK_RESPONSE_NONE': -1,
       'GTK_RESPONSE_REJECT': -2,
       'GTK_RESPONSE_ACCEPT': -3,
 'GTK_RESPONSE_DELETE_EVENT': -4,
           'GTK_RESPONSE_OK': -5,
       'GTK_RESPONSE_CANCEL': -6,
        'GTK_RESPONSE_CLOSE': -7,
          'GTK_RESPONSE_YES': -8,
           'GTK_RESPONSE_NO': -9,
        'GTK_RESPONSE_APPLY': -10,
         'GTK_RESPONSE_HELP': -11,
}

# Message Type
GTK_MESSAGE_INFO = Gtk.MessageType.INFO                   # 0
GTK_MESSAGE_WARNING = Gtk.MessageType.WARNING             # 1
GTK_MESSAGE_QUESTION = Gtk.MessageType.QUESTION           # 2
GTK_MESSAGE_ERROR = Gtk.MessageType.ERROR                 # 3
GTK_MESSAGE_OTHER = Gtk.MessageType.OTHER                 # 4

# Button Type
GTK_BUTTONS_NONE = Gtk.ButtonsType.NONE                   # 0
GTK_BUTTONS_OK = Gtk.ButtonsType.OK                       # 1
GTK_BUTTONS_CLOSE = Gtk.ButtonsType.CLOSE                 # 2
GTK_BUTTONS_CANCEL = Gtk.ButtonsType.CANCEL               # 3
GTK_BUTTONS_YES_NO = Gtk.ButtonsType.YES_NO               # 4
GTK_BUTTONS_OK_CANCEL = Gtk.ButtonsType.OK_CANCEL         # 5

# Response Type
GTK_RESPONSE_NONE = Gtk.ResponseType.NONE                 # -1
GTK_RESPONSE_REJECT = Gtk.ResponseType.REJECT             # -2
GTK_RESPONSE_ACCEPT = Gtk.ResponseType.ACCEPT             # -3
GTK_RESPONSE_DELETE_EVENT = Gtk.ResponseType.DELETE_EVENT # -4
GTK_RESPONSE_OK = Gtk.ResponseType.OK                     # -5
GTK_RESPONSE_CANCEL = Gtk.ResponseType.CANCEL             # -6
GTK_RESPONSE_CLOSE = Gtk.ResponseType.CLOSE               # -7
GTK_RESPONSE_YES = Gtk.ResponseType.YES                   # -8
GTK_RESPONSE_NO = Gtk.ResponseType.NO                     # -9
GTK_RESPONSE_APPLY = Gtk.ResponseType.APPLY               # -10
GTK_RESPONSE_HELP = Gtk.ResponseType.HELP                 # -11

######################################################################
# CLASS

def gdialog(dtext='Nothing.', label='label', title='dialog', dtype=GTK_MESSAGE_INFO, btype=GTK_BUTTONS_OK, markup=True):

    if dtype == GTK_MESSAGE_ERROR:
        print(f"{__name__}: {dtext}\n")
    else:
        print(f"{__name__}: {dtext}\n")

    dialog = Gtk.MessageDialog(
                               transient_for=Gtk.Window(),
                               flags=Gtk.DialogFlags.MODAL,
                               message_type=dtype,
                               buttons=btype,
                               #message_format=label
                               #text=label
                               )

    dialog.set_default_size(320, 256)
    dialog.set_title(title)
    dialog.set_markup(label)
    dialog.set_skip_taskbar_hint(False);
    dialog.set_position(Gtk.WindowPosition.CENTER);

    if markup is True:
        dialog.format_secondary_markup(dtext);
    else:
        dialog.format_secondary_text(dtext);

    # メッセージエリア内のGtkLabelを取得し、選択可能にする
    # GtkMessageDialog.get_message_area() は GtkBox を返す
    # その GtkBox の子ウィジェットが GtkLabel
    message_area = dialog.get_message_area()
    # GtkMessageDialog のメッセージラベルは、通常、message_area の最初の子（主メッセージ）です
    for child in message_area.get_children():
        if isinstance(child, Gtk.Label):
            child.set_selectable(True)
            #break # 最初に見つかったGtkLabelだけを対象とする場合

    retv = dialog.run()

    # クリップボードオブジェクトを取得
    clipboard = Gtk.Clipboard.get(Gdk.SELECTION_CLIPBOARD)
    # クリップボードの内容を永続化（ストア）
    # これにより、アプリケーションが終了してもクリップボードの内容が保持されます
    clipboard.store()

    dialog.destroy()
    # ^^;
    return retv

def gtxview(dtext='Nothing.', label='label', title='dialog', dtype=GTK_MESSAGE_INFO, btype=GTK_BUTTONS_OK, markup=True):

    def on_close(fixed, widget):
        widget.close()

    if dtype == GTK_MESSAGE_ERROR:
        print(f"{__name__}: {dtext}\n")
    else:
        print(f"{__name__}: {dtext}\n")

    gwin = Gtk.Window()

    dialog = Gtk.Dialog(
               title=title,
               transient_for=gwin,
               flags=Gtk.DialogFlags.MODAL,
               )

    dialog.set_default_size(384, 512)
    dialog.set_skip_taskbar_hint(False);
    dialog.set_position(Gtk.WindowPosition.CENTER);
    dialog.set_title(title)

    box = dialog.get_content_area()

    #attr = Pango.AttrList()
    #attr.insert(Pango.attr_foreground_new(65535, 0, 0))
    #attr.insert(Pango.attr_size_new(Pango.SCALE * 48))
    #attr.insert(Pango.attr_family_new('Serif'))

    #lbl1 = Gtk.Label(label, attr)
    lbl1 = Gtk.Label(label)
    lbl1.set_markup(f"<big><b>{label}</b></big>")
    lbl1.set_margin_top(6)
    lbl1.set_margin_bottom(6)
    box.add(lbl1)

    # GtkTextView を作成し、テキストをセット
    textview = Gtk.TextView()
    textbuffer = textview.get_buffer()
    textbuffer.set_text(dtext)

    # 読み取り専用にする
    textview.set_editable(True)
    textview.set_wrap_mode(True)
    textview.set_cursor_visible(True) # カーソルを非表示に
    textview.set_left_margin(4)
    textview.set_right_margin(4)
    textview.set_top_margin(4)
    textview.set_bottom_margin(4)

    # スクロール可能にするために GtkScrolledWindow に入れる
    scrolled_window = Gtk.ScrolledWindow()
    scrolled_window.set_vexpand(True)
    scrolled_window.set_hexpand(False)
    scrolled_window.add(textview)

    box.pack_start(scrolled_window, True, True, 2)

    btn1 = Gtk.Button("CLOSE")
    btn1.connect("clicked", on_close, dialog)
    box.add(btn1)
    dialog.show_all()

    retv = dialog.run()

    # クリップボードオブジェクトを取得
    clipboard = Gtk.Clipboard.get(Gdk.SELECTION_CLIPBOARD)
    # クリップボードの内容を永続化（ストア）
    # これにより、アプリケーションが終了してもクリップボードの内容が保持されます
    clipboard.store()

    dialog.destroy()
    # ^^;
    return retv

######################################################################
# MAIN
if __name__ == "__main__":
    print(f"[{__name__}]")
    print(__doc__)

#=====================================================================
# ALL - Make it directly accessible from the top level of the package
__all__ = [
    "gdialog", "gtxview", "MessageType", "ButtonType", "ResponseType", "GTK_MESSAGE_INFO", "GTK_MESSAGE_WARNING", "GTK_MESSAGE_QUESTION", "GTK_MESSAGE_ERROR", "GTK_MESSAGE_OTHER",
    "GTK_BUTTONS_NONE", "GTK_BUTTONS_OK", "GTK_BUTTONS_CLOSE", "GTK_BUTTONS_CANCEL", "GTK_BUTTONS_YES_NO", "GTK_BUTTONS_OK_CANCEL",
    "GTK_RESPONSE_NONE", "GTK_RESPONSE_REJECT", "GTK_RESPONSE_ACCEPT", "GTK_RESPONSE_DELETE_EVENT", "GTK_RESPONSE_OK", "GTK_RESPONSE_CANCEL", "GTK_RESPONSE_CLOSE", "GTK_RESPONSE_YES", "GTK_RESPONSE_NO", "GTK_RESPONSE_APPLY", "GTK_RESPONSE_HELP",
]

""" __DATA__

__END__ """



#!/home/twinkle/venv/bin/python

import os
import re
import json
import argparse

import urllib.parse

######################################################################
# LIBS

from sdgp import sdgp
from sdgp.Gtk import *

######################################################################
# ArgParse

def main():

    parser = argparse.ArgumentParser()
    parser.add_argument('-i', type=str, required=True)
    parser.add_argument('--textview', action="store_true", required=False)
    args = parser.parse_args()

    path = args.i
    flag = args.textview

    path = re.sub('^file://', '', path)
    path = urllib.parse.unquote(path)

    hako = sdgp(path)

    if hako is not None:

        # Pass Nothing
        if "prompt" in hako:
            prmp = hako.pop("prompt")
            #prmp = prmp.replace("<", "&lt;")
            #prmp = prmp.replace(">", "&gt;")
            #prmp = prmp.replace("\x22", "&quot;")
            prmp = prmp.replace("&", "&amp;")
            #prmp = prmp.replace("|", "\\|")
            prmp = re.sub("[\r\n]$", "", prmp)
        else: prmp = ""

        # Pass Nothing
        if "negativePrompt" in hako:
            ngtv = hako.pop("negativePrompt")
            #ngtv = ngtv.replace("<", "&lt;")
            #ngtv = ngtv.replace(">", "&gt;")
            #ngtv = ngtv.replace("\x22", "&quot;")
            ngtv = ngtv.replace("&", "&amp;")
            #ngtv = ngtv.replace("|", "\\|")
            ngtv = re.sub("[\r\n]$", "", prmp)
        else: ngtv = ""

        if flag is True:
            hall = f"Prompt: {prmp}\n\nNegative Prompt: {ngtv}\n\n"
            for k in hako.keys():
                hall = hall + f"{k}: {hako[k]}, "
            gtxview(hall, 'Stable Diffusion Creation Info', 'sd-get-prompt', GTK_MESSAGE_INFO, GTK_BUTTONS_OK, False)
        else:
            hall = f"<big><b>Prompt:</b></big> {prmp}\n\n<big><b>Negative Prompt:</b></big> {ngtv}\n\n"
            for k in hako.keys():
                hall = hall + f"<big><b>{k}:</b></big> {hako[k]}, "
            hall = hall.replace("&", "&amp;")
            gdialog(hall, 'Stable Diffusion Creation Info', 'sd-get-prompt', GTK_MESSAGE_INFO, GTK_BUTTONS_OK, True)

######################################################################
# MAIN
if __name__ == "__main__":
    main()

""" __DATA__

11 {'class_type': 'TripleCLIPLoader', 'inputs': {'clip_name1': 'clip_g_sdxl_base.safetensors', 'clip_name2': 'clip_l_sdxl_base.safetensors', 'clip_name3': 't5xxl.safetensors'}, '_properties': {'Node name for S&R': 'TripleCLIPLoader'}}
13 {'class_type': 'ModelSamplingSD3', 'inputs': {'model': ['252', 0], 'shift': 3.0}, '_properties': {'Node name for S&R': 'ModelSamplingSD3'}}
135 {'class_type': 'EmptySD3LatentImage', 'inputs': {'batch_size': 1, 'height': 1024, 'width': 1024}, '_properties': {'Node name for S&R': 'EmptySD3LatentImage'}}
231 {'class_type': 'VAEDecode', 'inputs': {'samples': ['271', 0], 'vae': ['252', 2]}, '_properties': {'Node name for S&R': 'VAEDecode'}}
233 {'class_type': 'PreviewImage', 'inputs': {'images': ['231', 0]}, '_properties': {'Node name for S&R': 'PreviewImage'}}
252 {'class_type': 'CheckpointLoaderSimple', 'inputs': {'_base_model': 'SD 3', 'ckpt_name': 'EMS-398059-EMS.safetensors', 'tams_ckpt_name': 'stable-diffusion-3 - medium'}, '_properties': {'Node name for S&R': 'CheckpointLoaderSimple'}}
271 {'class_type': 'KSampler', 'inputs': {'cfg': 4.5, 'denoise': 1.0, 'latent_image': ['135', 0], 'model': ['13', 0], 'negative': ['69', 0], 'positive': ['6', 0], 'sampler_name': 'dpmpp_2m', 'scheduler': 'sgm_uniform', 'seed': 813265401979403, 'steps': 28}, '_properties': {'Node name for S&R': 'KSampler'}}
6 {'class_type': 'CLIPTextEncode', 'inputs': {'clip': ['11', 0], 'text': "a female character with long, flowing hair that appears to be made of ethereal, swirling patterns resembling the Northern Lights or Aurora Borealis. The background is dominated by deep blues and purples, creating a mysterious and dramatic atmosphere. The character's face is serene, with pale skin and striking features. She wears a dark-colored outfit with subtle patterns. The overall style of the artwork is reminiscent of fantasy or supernatural genres"}, '_properties': {'Node name for S&R': 'CLIPTextEncode'}}
67 {'class_type': 'ConditioningZeroOut', 'inputs': {'conditioning': ['71', 0]}, '_properties': {'Node name for S&R': 'ConditioningZeroOut'}}
68 {'class_type': 'ConditioningSetTimestepRange', 'inputs': {'conditioning': ['67', 0], 'end': 1.0, 'start': 0.1}, '_properties': {'Node name for S&R': 'ConditioningSetTimestepRange'}}
69 {'class_type': 'ConditioningCombine', 'inputs': {'conditioning_1': ['68', 0], 'conditioning_2': ['70', 0]}, '_properties': {'Node name for S&R': 'ConditioningCombine'}}
70 {'class_type': 'ConditioningSetTimestepRange', 'inputs': {'conditioning': ['71', 0], 'end': 0.1, 'start': 0.0}, '_properties': {'Node name for S&R': 'ConditioningSetTimestepRange'}}
71 {'class_type': 'CLIPTextEncode', 'inputs': {'clip': ['11', 0], 'text': 'bad quality, poor quality, doll, disfigured, jpg, toy, bad anatomy, missing limbs, missing fingers, 3d, cgi'}, '_properties': {'Node name for S&R': 'CLIPTextEncode'}}

__END__ """



from setuptools import setup, find_packages

setup(
    name='sdgp',
    version='0.96.0',
    packages=find_packages(exclude=['tests']),
    entry_points={
        'console_scripts': [
            'my_command = sdgp.module:main_func',
        ],
    },
    # ... 他のメタデータ ...
)
